#include <psptypes.h>
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <pspdisplay.h>
#include <pspctrl.h>
#include "graphics.h"
#include "error.h"
#include "font.h"
#include "dialog.h"
#include "input.h"
#include "textbox.h"
#include "main.h"
#include "fileBrowser.h"
#include "textField.h"
#include "filesys.h"
#include "settings.h"

Image* dlgOverlay = NULL;
char dlgScreenshoting = 0;

void dlgRenderLoop() {
     if(dlgOverlay == NULL) {
          dlgOverlay = imageCreate(SCREEN_WIDTH, SCREEN_HEIGHT);
     }
     renderLoop();
     imageDrawAlpha(0, 0, dlgOverlay);
}

unsigned char dlgConfirm(char* inLineA, char* inLineB) {
     char tempString[256];
     int tempWidth = MAX(strlen(inLineA), strlen(inLineB));
     tempWidth *= (defaultFont->fontWidth + 1);
     tempWidth += 16;
     fontSetColor(menuForeColor);

     while(1) {
          inputPoll();
          sceDisplayWaitVblankStart();
          graphicsCopyBuffers();
          screenDrawFilledRect((SCREEN_WIDTH >> 1) - (tempWidth >> 1), (SCREEN_HEIGHT >> 1) - 32, tempWidth, 64, menuBackColor);
          screenDrawRect((SCREEN_WIDTH >> 1) - (tempWidth >> 1), (SCREEN_HEIGHT >> 1) - 32, tempWidth, 64, menuBorderColor);

          fontPrintCenter(((SCREEN_HEIGHT >> 1) - 24), inLineA, defaultFont);
          fontPrintCenter(((SCREEN_HEIGHT >> 1) - 12), inLineB, defaultFont);

          sprintf(tempString, "[%c] Ok [%c] Cancel", ASCII_CROSS, ASCII_CIRCLE);
          fontPrintCenter(((SCREEN_HEIGHT >> 1) + 16), tempString, defaultFont);
          
          if(inputRepeat[INPUT_CROSS] == 1) {
               inputPoll();
               return 1;
          }
          if(inputRepeat[INPUT_CIRCLE] == 1) {
               inputPoll();
               return 0;
          }
          
          graphicsSwapBuffers();
     }
}

unsigned char dlgConfirmClear() {
     char tempString[256];
     fontSetColor(menuForeColor);

     while(1) {
          inputPoll();
          sceDisplayWaitVblankStart();
          graphicsCopyBuffers();
          screenDrawFilledRect((SCREEN_WIDTH >> 1) - 64, (SCREEN_HEIGHT >> 1) - 32, 128, 64, menuBackColor);
          screenDrawRect((SCREEN_WIDTH >> 1) - 64, (SCREEN_HEIGHT >> 1) - 32, 128, 64, menuBorderColor);

          sprintf(tempString, "All unsaved data");
          fontPrintCenter(((SCREEN_HEIGHT >> 1) - 24), tempString, defaultFont);
          sprintf(tempString, "will be lost.");
          fontPrintCenter(((SCREEN_HEIGHT >> 1) - 12), tempString, defaultFont);

          sprintf(tempString, "[%c] Ok [%c] Cancel", ASCII_CROSS, ASCII_CIRCLE);
          fontPrintCenter(((SCREEN_HEIGHT >> 1) + 16), tempString, defaultFont);
          
          if(inputRepeat[INPUT_CROSS] == 1) {
               inputPoll();
               return 1;
          }
          if(inputRepeat[INPUT_CIRCLE] == 1) {
               inputPoll();
               return 0;
          }
          
          graphicsSwapBuffers();
     }
}

unsigned char dlgConfirmOverwrite() {
     char tempString[256];
     fontSetColor(menuForeColor);

     while(1) {
          inputPoll();
          sceDisplayWaitVblankStart();
          graphicsCopyBuffers();
          screenDrawFilledRect((SCREEN_WIDTH >> 1) - 96, (SCREEN_HEIGHT >> 1) - 32, 192, 64, menuBackColor);
          screenDrawRect((SCREEN_WIDTH >> 1) - 96, (SCREEN_HEIGHT >> 1) - 32, 192, 64, menuBorderColor);

          sprintf(tempString, "A file of this name");
          fontPrintCenter(((SCREEN_HEIGHT >> 1) - 24), tempString, defaultFont);
          sprintf(tempString, "already exists.");
          fontPrintCenter(((SCREEN_HEIGHT >> 1) - 12), tempString, defaultFont);
          sprintf(tempString, "Overwrite?");
          fontPrintCenter((SCREEN_HEIGHT >> 1), tempString, defaultFont);

          sprintf(tempString, "[%c] Ok [%c] Cancel", ASCII_CROSS, ASCII_CIRCLE);
          fontPrintCenter(((SCREEN_HEIGHT >> 1) + 16), tempString, defaultFont);
          
          if(inputRepeat[INPUT_CROSS] == 1) {
               inputPoll();
               return 1;
          }
          if(inputRepeat[INPUT_CIRCLE] == 1) {
               inputPoll();
               return 0;
          }
          
          graphicsSwapBuffers();
     }
}

unsigned char dlgConfirmDelete(char* inPath) {
     char tempString[256];
     char* tempFileName = filesysFileNameFromPath(inPath);
     fontSetColor(menuForeColor);

     while(1) {
          inputPoll();
          sceDisplayWaitVblankStart();
          graphicsCopyBuffers();
          screenDrawFilledRect((SCREEN_WIDTH >> 1) - 96, (SCREEN_HEIGHT >> 1) - 32, 192, 64, menuBackColor);
          screenDrawRect((SCREEN_WIDTH >> 1) - 96, (SCREEN_HEIGHT >> 1) - 32, 192, 64, menuBorderColor);
          
          if(filesysIsFile(inPath) > 0)
               sprintf(tempString, "Really delete file");
          else
               sprintf(tempString, "Really delete folder");
          fontPrintCenter(((SCREEN_HEIGHT >> 1) - 24), tempString, defaultFont);
          sprintf(tempString, "named:");
          fontPrintCenter(((SCREEN_HEIGHT >> 1) - 12), tempString, defaultFont);

          fontPrintCenter((SCREEN_HEIGHT >> 1), tempFileName, defaultFont);

          sprintf(tempString, "[%c] Ok [%c] Cancel", ASCII_CROSS, ASCII_CIRCLE);
          fontPrintCenter(((SCREEN_HEIGHT >> 1) + 16), tempString, defaultFont);
          
          if(inputRepeat[INPUT_CROSS] == 1) {
               free(tempFileName);
               inputPoll();
               return 1;
          }
          if(inputRepeat[INPUT_CIRCLE] == 1) {
               free(tempFileName);
               inputPoll();
               return 0;
          }
          
          graphicsSwapBuffers();
     }
}

unsigned char dlgHelp(char* inFile) {
     char tempString[256];
     fontSetColor(menuForeColor);

     while(1) {
          inputPoll();
          sceDisplayWaitVblankStart();
          graphicsCopyBuffers();
          screenDrawFilledRect((SCREEN_WIDTH >> 1) - 112, (SCREEN_HEIGHT >> 1) - 32, 224, 64, menuBackColor);
          screenDrawRect((SCREEN_WIDTH >> 1) - 112, (SCREEN_HEIGHT >> 1) - 32, 224, 64, menuBorderColor);

          sprintf(tempString, "This will load %s.", inFile);
          fontPrintCenter(((SCREEN_HEIGHT >> 1) - 24), tempString, defaultFont);
          sprintf(tempString, "Any unsaved data will be lost.");
          fontPrintCenter(((SCREEN_HEIGHT >> 1) - 12), tempString, defaultFont);
          sprintf(tempString, "Continue?");
          fontPrintCenter((SCREEN_HEIGHT >> 1), tempString, defaultFont);

          sprintf(tempString, "[%c] Ok [%c] Cancel", ASCII_CROSS, ASCII_CIRCLE);
          fontPrintCenter(((SCREEN_HEIGHT >> 1) + 16), tempString, defaultFont);
          
          if(inputRepeat[INPUT_CROSS] == 1) {
               inputPoll();
               return 1;
          }
          if(inputRepeat[INPUT_CIRCLE] == 1) {
               inputPoll();
               return 0;
          }
          
          graphicsSwapBuffers();
     }
}

void dlgFileError(char* inFile) {
     char tempString[256];
     fontSetColor(menuForeColor);

     while(1) {
          inputPoll();
          sceDisplayWaitVblankStart();
          graphicsCopyBuffers();
          screenDrawFilledRect((SCREEN_WIDTH >> 1) - 128, (SCREEN_HEIGHT >> 1) - 32, 256, 64, menuBackColor);
          screenDrawRect((SCREEN_WIDTH >> 1) - 128, (SCREEN_HEIGHT >> 1) - 32, 256, 64, menuBorderColor);

          sprintf(tempString, "Couldn't open file:");
          fontPrintCenter(((SCREEN_HEIGHT >> 1) - 24), tempString, defaultFont);
          fontPrintCenter(((SCREEN_HEIGHT >> 1) - 12), inFile, defaultFont);

          sprintf(tempString, "[%c/%c] Close", ASCII_CROSS, ASCII_CIRCLE);
          fontPrintCenter(((SCREEN_HEIGHT >> 1) + 16), tempString, defaultFont);
          
          if((inputRepeat[INPUT_CROSS] == 1) || (inputRepeat[INPUT_CIRCLE] == 1)) {
               inputPoll();
               return;
          }
          
          graphicsSwapBuffers();
     }
}

void dlgMessage(char* inMessage) {
     char tempString[256];
     fontSetColor(menuForeColor);

     while(1) {
          inputPoll();
          sceDisplayWaitVblankStart();
          graphicsCopyBuffers();
          screenDrawFilledRect((SCREEN_WIDTH >> 1) - 128, (SCREEN_HEIGHT >> 1) - 32, 256, 56, menuBackColor);
          screenDrawRect((SCREEN_WIDTH >> 1) - 128, (SCREEN_HEIGHT >> 1) - 32, 256, 56, menuBorderColor);

          fontPrintCenter(((SCREEN_HEIGHT >> 1) - 24), inMessage, defaultFont);

          sprintf(tempString, "[%c/%c] Close", ASCII_CROSS, ASCII_CIRCLE);
          fontPrintCenter(((SCREEN_HEIGHT >> 1) + 4), tempString, defaultFont);
          
          if((inputRepeat[INPUT_CROSS] == 1) || (inputRepeat[INPUT_CIRCLE] == 1)) {
               inputPoll();
               return;
          }
          
          graphicsSwapBuffers();
     }
}

char* dlgTextEntry(char* inPrompt) {
     char tempString[256];
     Keyboard* dlgKeyb = keybCreate((SCREEN_WIDTH >> 1) - 60, (SCREEN_HEIGHT >> 1) + 10);
     dlgKeyb->keybFocus = !0;
     TextField* txtfTextEntry = txtfCreate((SCREEN_WIDTH >> 1) - 120, (SCREEN_HEIGHT >> 1) - 14, 34, defaultFont);
     unsigned char tempChar;
     fontSetColor(menuForeColor);

     while(1) {
          inputPoll();
          keybTick(dlgKeyb);
          
          sceDisplayWaitVblankStart();
          graphicsCopyBuffers();
          screenDrawFilledRect((SCREEN_WIDTH >> 1) - 128, (SCREEN_HEIGHT >> 1) - 32, 256, 112, menuBackColor);
          screenDrawRect((SCREEN_WIDTH >> 1) - 128, (SCREEN_HEIGHT >> 1) - 32, 256, 112, menuBorderColor);

          txtfDraw(txtfTextEntry);
          fontPrintCenter(((SCREEN_HEIGHT >> 1) - 24), inPrompt, defaultFont);
          
          keybDraw(dlgKeyb);
          tempChar = keybBufferPop(dlgKeyb);
          while(tempChar != 0) {
               if(tempChar == ASCII_BS) {
                   txtfBackSpace(txtfTextEntry);
               } else if(tempChar == ASCII_LF) {
                    keybFree(dlgKeyb);
                     return (char*)txtfFree(txtfTextEntry);
               } else {
                   txtfAddChar(txtfTextEntry, tempChar);
               }
               tempChar = keybBufferPop(dlgKeyb);
          }
          
          
          sprintf(tempString, "[%c/Enter] Accept  [%c] Close", ASCII_UP, ASCII_DOWN);
          fontPrintCenter(((SCREEN_HEIGHT >> 1) + 62), tempString, defaultFont);
          if(inputRepeat[INPUT_UP] == 1) {
               keybFree(dlgKeyb);
               inputPoll();
               return (char*)txtfFree(txtfTextEntry);
          } else if(inputRepeat[INPUT_DOWN] == 1) {
               inputPoll();
               keybFree(dlgKeyb);
               free(txtfFree(txtfTextEntry));
               return NULL;
          }
          
          graphicsSwapBuffers();
     }
}

int dlgInsertSymbol() {
     char tempString[256];
     unsigned char tempCharSel = 65;
     int tempInt = tempCharSel;
     fontSetColor(menuForeColor);

     while(1) {
          inputPoll();
          sceDisplayWaitVblankStart();
          graphicsCopyBuffers();
          screenDrawFilledRect((SCREEN_WIDTH >> 1) - 96, (SCREEN_HEIGHT >> 1) - 32, 192, 64, menuBackColor);
          screenDrawRect((SCREEN_WIDTH >> 1) - 96, (SCREEN_HEIGHT >> 1) - 32, 192, 64, menuBorderColor);

          sprintf(tempString, "Insert Symbol");
          fontPrintCenter(((SCREEN_HEIGHT >> 1) - 24), tempString, defaultFont);
          sprintf(tempString, "Symbol: %i [ ]", tempInt);
          fontPrint(187, ((SCREEN_HEIGHT >> 1) - 12), tempString, defaultFont);
          fontPrintChar(187 + ((strlen(tempString) - 2) * (defaultFont->fontWidth + 1)), ((SCREEN_HEIGHT >> 1) - 12), tempCharSel, defaultFont);

          sprintf(tempString, "[%c] Select [%c] Close", ASCII_CROSS, ASCII_CIRCLE);
          fontPrintCenter(((SCREEN_HEIGHT >> 1) + 16), tempString, defaultFont);
          
          if((inputRepeat[INPUT_UP] == 1) || (inputRepeat[INPUT_UP] > INPUT_REPEATRATE)) {
               tempInt++;
               if(tempInt > 255)
                    tempInt -= 256;
               tempCharSel = tempInt;
          } else if((inputRepeat[INPUT_DOWN] == 1) || (inputRepeat[INPUT_DOWN] > INPUT_REPEATRATE)) {
               tempInt--;
               if(tempInt < 0)
                    tempInt += 256;
               tempCharSel = tempInt;
          } else if((inputRepeat[INPUT_LEFT] == 1) || (inputRepeat[INPUT_LEFT] > INPUT_REPEATRATE)) {
               tempInt -= 10;
               if(tempInt < 0)
                    tempInt += 256;
               tempCharSel = tempInt;
          } else if((inputRepeat[INPUT_RIGHT] == 1) || (inputRepeat[INPUT_RIGHT] > INPUT_REPEATRATE)) {
               tempInt += 10;
               if(tempInt > 255)
                    tempInt -= 256;
               tempCharSel = tempInt;
          }
          
          if((inputRepeat[INPUT_CIRCLE] == 1)) {
               inputPoll();
               return -1;
          } else if((inputRepeat[INPUT_CROSS] == 1)) {
               inputPoll();
               return tempInt;
          }
          
          graphicsSwapBuffers();
     }
}

int dlgGotoLine() {
     if(txtbMain->txtbLineCount <= 1) {
           dlgMessage("Only 1 line to goto.");
           return 0;
     }
     char tempString[256];
     int tempInt = 1;
     fontSetColor(menuForeColor);

     while(1) {
          inputPoll();
          sceDisplayWaitVblankStart();
          graphicsCopyBuffers();
          screenDrawFilledRect((SCREEN_WIDTH >> 1) - 96, (SCREEN_HEIGHT >> 1) - 32, 192, 64, menuBackColor);
          screenDrawRect((SCREEN_WIDTH >> 1) - 96, (SCREEN_HEIGHT >> 1) - 32, 192, 64, menuBorderColor);

          sprintf(tempString, "Goto Line");
          fontPrintCenter(((SCREEN_HEIGHT >> 1) - 24), tempString, defaultFont);
          sprintf(tempString, "Line: %i", tempInt);
          fontPrintCenter(((SCREEN_HEIGHT >> 1) - 12), tempString, defaultFont);

          sprintf(tempString, "[%c] Goto [%c] Close", ASCII_CROSS, ASCII_CIRCLE);
          fontPrintCenter(((SCREEN_HEIGHT >> 1) + 16), tempString, defaultFont);
          
          if((inputRepeat[INPUT_UP] == 1) || (inputRepeat[INPUT_UP] > INPUT_REPEATRATE)) {
               tempInt++;
               while(tempInt > txtbMain->txtbLineCount)
                    tempInt -= txtbMain->txtbLineCount;
          } else if((inputRepeat[INPUT_DOWN] == 1) || (inputRepeat[INPUT_DOWN] > INPUT_REPEATRATE)) {
               tempInt--;
               while(tempInt <= 0)
                    tempInt += txtbMain->txtbLineCount;
          } else if((inputRepeat[INPUT_LEFT] == 1) || (inputRepeat[INPUT_LEFT] > INPUT_REPEATRATE)) {
               tempInt -= 10;
               while(tempInt <= 0)
                    tempInt += txtbMain->txtbLineCount;
          } else if((inputRepeat[INPUT_RIGHT] == 1) || (inputRepeat[INPUT_RIGHT] > INPUT_REPEATRATE)) {
               tempInt += 10;
               if(tempInt > txtbMain->txtbLineCount)
                    tempInt -= txtbMain->txtbLineCount;
          }
          
          if((inputRepeat[INPUT_CIRCLE] == 1)) {
               inputPoll();
               return -1;
          } else if((inputRepeat[INPUT_CROSS] == 1)) {
               inputPoll();
               return tempInt;
          }
          
          graphicsSwapBuffers();
     }
}

void dlgScreenshot() {
     if(dlgScreenshoting != 0)
          return;
     dlgScreenshoting = 1;
     
     Image* dlgBack = imageFromScreen();
     Image* dlgThumb = imageThumbnail(dlgBack);
     char tempString[256];
     fontSetColor(menuForeColor);

     while(1) {
          inputPoll();
          sceDisplayWaitVblankStart();
          imageDraw(0, 0, dlgBack);
          screenDrawFilledRect((SCREEN_WIDTH >> 1) - 128, (SCREEN_HEIGHT >> 1) - 96, 256, 192, menuBackColor);
          screenDrawRect((SCREEN_WIDTH >> 1) - 128, (SCREEN_HEIGHT >> 1) - 96, 256, 192, menuBorderColor);
          
          imageDrawCenter(dlgThumb);

          sprintf(tempString, "Save Screenshot?");
          fontPrintCenter(((SCREEN_HEIGHT >> 1) - 84), tempString, defaultFont);

          sprintf(tempString, "[%c] Ok [%c] Cancel", ASCII_CROSS, ASCII_CIRCLE);
          fontPrintCenter(((SCREEN_HEIGHT >> 1) + 76), tempString, defaultFont);
          
          if(inputRepeat[INPUT_CROSS] == 1) {
               if(filesysIsFile("Screenshots") != 0)
                    filesysMkdir("Screenshots");
               int tempInt = 0;
               char tempString[256];
               sprintf(tempString, "screenshots/Screenshot%i.png", tempInt);
               while(filesysFileExists(tempString) != 0) {
                    tempInt++;
                    sprintf(tempString, "screenshots/Screenshot%i.png", tempInt);
               }
               imageSave(tempString, dlgBack, 0);
               
               imageFree(dlgThumb);
               imageFree(dlgBack);
               
               sprintf(tempString, "Screenshot saved.");
               dlgMessage(tempString);
               inputPoll();
               dlgScreenshoting = 0;
               return;
          }
          if(inputRepeat[INPUT_CIRCLE] == 1) {
               imageFree(dlgBack);
               imageFree(dlgThumb);
               inputPoll();
               dlgScreenshoting = 0;
               return;
          }
          
          graphicsSwapBuffers();
     }
}

void dlgFind() {
     Image* dlgBack = imageFromScreen();
     char tempString[256];
     char* tempFind = dlgTextEntry("Find:");
     if(tempFind == NULL) {
          return;
     } else if(tempFind[0] == 0) {
          free(tempFind);
          return;
     }
     int tempInt = txtbFindNext(txtbMain, tempFind);
     if(tempInt < 0)
          tempInt = txtbFindPrev(txtbMain, tempFind);

     while(1) {
          inputPoll();
          sceDisplayWaitVblankStart();
          imageDraw(0, 0, dlgBack);
          fontSetColor(menuForeColor);
          
          screenDrawFilledRect((SCREEN_WIDTH >> 1) - 128, (SCREEN_HEIGHT >> 1) - 32, 256, 64, menuBackColor);
          screenDrawRect((SCREEN_WIDTH >> 1) - 128, (SCREEN_HEIGHT >> 1) - 32, 256, 64, menuBorderColor);

          sprintf(tempString, "Find: %s", tempFind);
          fontPrintCenter(((SCREEN_HEIGHT >> 1) - 24), tempString, defaultFont);
          if(tempInt < 0)
               sprintf(tempString, "Not found");
          else
               sprintf(tempString, "Found on line: %i", (tempInt - 1));
          fontPrintCenter(((SCREEN_HEIGHT >> 1) - 12), tempString, defaultFont);

          sprintf(tempString, "[%c/%c] Find Next/Prev [%c/%c] Close",ASCII_UP, ASCII_DOWN, ASCII_CROSS, ASCII_CIRCLE);
          fontPrintCenter(((SCREEN_HEIGHT >> 1) + 16), tempString, defaultFont);
          
          if((inputRepeat[INPUT_UP] == 1) || (inputRepeat[INPUT_UP] > INPUT_REPEATRATE)) {
               tempInt = txtbFindPrev(txtbMain, tempFind);
          } else if((inputRepeat[INPUT_DOWN] == 1) || (inputRepeat[INPUT_DOWN] > INPUT_REPEATRATE)) {
               tempInt = txtbFindNext(txtbMain, tempFind);
          }
          if((inputRepeat[INPUT_CIRCLE] == 1) || (inputRepeat[INPUT_CROSS] == 1)) {
               free(tempFind);
               imageFree(dlgBack);
               inputPoll();
               return;
          }
          
          graphicsSwapBuffers();
     }
}

void dlgReplace() {
     Image* dlgBack = imageFromScreen();
     char tempString[256];
     char* tempFind = dlgTextEntry("Find:");
     fontSetColor(menuForeColor);
     if(tempFind == NULL) {
          return;
     } else  if(tempFind[0] == 0) {
          free(tempFind);
          return;
     }
     char* tempReplace = dlgTextEntry("Replace:");
     if(tempReplace == NULL) {
          free(tempFind);
          return;
     } else if(tempReplace[0] == 0) {
          free(tempReplace);
          free(tempFind);
          return;
     }

     while(1) {
          inputPoll();
          sceDisplayWaitVblankStart();
          imageDraw(0, 0, dlgBack);
          screenDrawFilledRect((SCREEN_WIDTH >> 1) - 128, (SCREEN_HEIGHT >> 1) - 32, 256, 64, menuBackColor);
          screenDrawRect((SCREEN_WIDTH >> 1) - 128, (SCREEN_HEIGHT >> 1) - 32, 256, 64, menuBorderColor);

          sprintf(tempString, "Replace: %s", tempFind);
          fontPrintCenter(((SCREEN_HEIGHT >> 1) - 24), tempString, defaultFont);
          sprintf(tempString, "With: %s", tempReplace);
          fontPrintCenter(((SCREEN_HEIGHT >> 1) - 12), tempString, defaultFont);

          sprintf(tempString, "[%c] Replace All  [%c] Close", ASCII_CROSS, ASCII_CIRCLE);
          fontPrintCenter(((SCREEN_HEIGHT >> 1) + 16), tempString, defaultFont);
          
          if(inputRepeat[INPUT_CROSS] == 1) {
               txtbReplace(txtbMain, tempFind, tempReplace);
               break;
          }
          if(inputRepeat[INPUT_CIRCLE] == 1) 
               break;
          
          graphicsSwapBuffers();
     }
     free(tempFind);
     free(tempReplace);
     imageFree(dlgBack);
     inputPoll();
     return;
}

void dlgCredits() {
     char tempString[256];
     sprintf(tempString, "skins/%s/splash.png", skinPath);
     Image* dlgBack = imageLoad(tempString);
     char* tempCredits = filesysLoad("CREDITS");
     fontSetColor(menuForeColor);

     fontSetColor(0xFFFFFFFF);

     while(1) {
          inputPoll();
          sceDisplayWaitVblankStart();
          imageDraw(0, 0, dlgBack);

          fontPrintCenterMultiline(2, tempCredits, defaultFont);

          sprintf(tempString, "[%c/%c] Close", ASCII_CROSS, ASCII_CIRCLE);
          fontPrintCenter((SCREEN_HEIGHT - defaultFont->fontHeight - 2), tempString, defaultFont);
          
          if((inputRepeat[INPUT_CIRCLE] == 1) || (inputRepeat[INPUT_CROSS] == 1)) 
               break;
          
          graphicsSwapBuffers();
     }
     fontSetColor(menuBorderColor);
     imageFree(dlgBack);
     inputPoll();
     return;
}

void dlgAbout() {
     char tempString[256];
     sprintf(tempString, "skins/%s/about.png", skinPath);
     Image* dlgImage = imageLoad(tempString);
     fontSetColor(menuForeColor);

     
     while(1) {
          inputPoll();
          sceDisplayWaitVblankStart();
          graphicsCopyBuffers();
          
          screenDrawFilledRect((SCREEN_WIDTH >> 1) - 80, (SCREEN_HEIGHT >> 1) - 64, 160, 128, menuBackColor);
          screenDrawRect((SCREEN_WIDTH >> 1) - 80, (SCREEN_HEIGHT >> 1) - 64, 160, 128, menuBorderColor);
          imageDrawCenter(dlgImage);
          fontPrintCenter((SCREEN_HEIGHT >> 1) - 56, "About", defaultFont);
          sprintf(tempString, "[%c/%c] Close", ASCII_CROSS, ASCII_CIRCLE);
          fontPrintCenter((SCREEN_HEIGHT >> 1) + 48, tempString, defaultFont);
          
          if((inputRepeat[INPUT_CROSS] == 1) || (inputRepeat[INPUT_CIRCLE] == 1))
               return;
          
          graphicsSwapBuffers();
     }
     free(dlgImage);
}
