#include <pspctrl.h>
#include <string.h>
#include <malloc.h>
#include <stdio.h>
#include <pspdisplay.h>
#include <pspkernel.h>
#include "filesys.h"
#include "error.h"
#include "dialog.h"

int filesysFileCount = 0;
int filesysFolderCount = 0;
filesysEntry* filesysFiles = NULL;
filesysEntry* filesysFolders = NULL;

int filesysDir(char* inPath) {
     char tempString[768];
     if(filesysFiles != NULL)
          free(filesysFiles);
     if(filesysFolders != NULL)
          free(filesysFolders);
     filesysFileCount = 0;
     filesysFolderCount = 0;

     struct SceIoDirent tempDir; 
     memset(&tempDir, 0, sizeof(SceIoDirent)); 
	
     static int tempFolder;
     tempFolder = sceIoDopen(inPath);

     if(tempFolder > 0) {
          while(sceIoDread(tempFolder, &tempDir) > 0) {
               sprintf(tempString, "%s/%s", inPath, tempDir.d_name);
               if(filesysIsFile(tempString) == 1) {
                    filesysFileCount++;
               } else if((filesysIsFile(tempString) == 0) || (filesysIsFile(tempString) == 2)){
                    filesysFolderCount++;
               }
          }
          sceIoDclose(tempFolder);
          if(filesysFileCount > 0) {
               filesysFiles = (filesysEntry*) malloc(filesysFileCount * sizeof(filesysEntry));
               if(filesysFiles == NULL) {
                    sprintf(tempString, "Couldn't create filesysEntry struct.\nOut of memory.");
                    errorThrow(tempString);
                    filesysFiles = 0;
                    filesysFolders = 0;
                    return -1;
               }
          }
          if(filesysFolderCount > 0) {
               filesysFolders = (filesysEntry*) malloc(filesysFolderCount * sizeof(filesysEntry));
               if(filesysFolders == NULL) {
                    sprintf(tempString, "Couldn't create filesysDirectory sub-struct.\nOut of memory.");
                    errorThrow(tempString);
                    if(filesysFiles != NULL)
                         free(filesysFiles);
                    filesysFiles = 0;
                    filesysFolders = 0;
                    return -1;
               }
          }
          unsigned int tempFolderPtr = 0;
          unsigned int tempFilePtr = 0;
          tempFolder = sceIoDopen(inPath);
          if(tempFolder <= 0) {
               sprintf(tempString, "Can't open directory descriptor.");
               errorThrow(tempString);
               if(filesysFiles != NULL)
                    free(filesysFiles);
               if(filesysFolders != NULL)
                    free(filesysFolders);
               filesysFiles = 0;
               filesysFolders = 0;
               return -1;     
          }
          while(sceIoDread(tempFolder, &tempDir) > 0) { 
               sprintf(tempString, "%s/%s", inPath, tempDir.d_name);
               if(filesysIsFile(tempString) == 1) {
                    strcpy(filesysFiles[tempFilePtr].fseName, tempDir.d_name);
                    tempFilePtr++;
               } else if((filesysIsFile(tempString) == 0) || (filesysIsFile(tempString) == 2)) {
                    strcpy(filesysFolders[tempFolderPtr].fseName, tempDir.d_name);
                    tempFolderPtr++;
               }
          }
     } else {
          sprintf(tempString, "Can't open directory descriptor.");
          errorThrow(tempString);
          return -1;
     }
     sceIoDclose(tempFolder);
     return 1;
}

char filesysIsFile(char* inPath) { 
     char* tempFileName = filesysFileNameFromPath(inPath);
     if(tempFileName[0] == 0) {
          free(tempFileName);                   
          return -1;
     }
     if(strcmp(tempFileName, "..") == 0) {
          free(tempFileName);                         
          return 2;
     } else if(strcmp(tempFileName, ".") == 0) {
          free(tempFileName);                         
          return 3;
     }
     free(tempFileName);
     
     SceIoStat tempStats;
     sceIoGetstat(inPath, &tempStats);

     if(tempStats.st_mode & FIO_S_IFDIR) 
          return 0; 
     else 
          return 1;  
}

char filesysFileExists(char* inPath) {
     FILE* tempFile = fopen(inPath, "rb");
     if(tempFile != NULL) {
          fclose(tempFile);
          return 1;
     }
     return 0;
}

char* filesysFileNameFromPath(char* inPath) {
     int tempPtr = 0;
     char tempIsPath = 0;
     while(inPath[tempPtr] != 0) {
          tempPtr++;
          if(inPath[tempPtr] ==  47)
               tempIsPath = 1;
     }
     char* tempOut = (char*) malloc(129 * sizeof(char));
     if(tempIsPath != 0) {
          while(inPath[tempPtr - 1] != 47)
               tempPtr--;
          strcpy(tempOut, &inPath[tempPtr]);
          tempOut[strlen(&inPath[tempPtr])] = 0;
     } else {
          strcpy(tempOut, inPath);
     }
     return tempOut;
}

void filesysMkdir(char* inDir) {
     sceIoMkdir(inDir, 0777);     
}

void filesysDelete(char* inPath) {
     int tempIsFile = filesysIsFile(inPath);     
     if(tempIsFile > 0)
           sceIoRemove(inPath);
     if(tempIsFile != 0)
          return;
     filesysEntry* tempFileList;
     int tempFileListSize = 0;
     struct SceIoDirent tempDir; 
     memset(&tempDir, 0, sizeof(SceIoDirent)); 
     int tempFolder;
     tempFolder = sceIoDopen(inPath);
     if(tempFolder <= 0) {
          errorThrow("Can't open directory descriptor.\nProgram still stable.");
          return;
     }              
     while(sceIoDread(tempFolder, &tempDir) > 0) {
          tempFileListSize++;                                  
     }
     sceIoDclose(tempFolder);
     tempFileList = (filesysEntry*) malloc(tempFileListSize * sizeof(filesysEntry));
     if(tempFileList == NULL) {
          errorThrow("Couldn't create directory listing for\ndelete operation.\nProgram still stable.");
          return;
     }
     tempFolder = sceIoDopen(inPath);
     if(tempFolder <= 0) {
          errorThrow("Can't open directory descriptor.\nProgram still stable.");
          free(tempFileList);
          return;
     }
     int tempPtr = 0;
     while(sceIoDread(tempFolder, &tempDir) > 0) {                             
          sprintf(tempFileList[tempPtr].fseName, "%s/%s", inPath, tempDir.d_name);
          tempPtr++;
          if(tempPtr >= tempFileListSize)
               break;
     }
     sceIoDclose(tempFolder);
     for(tempPtr = 0; tempPtr < tempFileListSize; tempPtr++) {
          if((filesysIsFile(tempFileList[tempPtr].fseName) >= 0) || (filesysIsFile(tempFileList[tempPtr].fseName) <= 1))
               filesysDelete(tempFileList[tempPtr].fseName);
     }
     free(tempFileList);
     if(sceIoRmdir(inPath) < 0) {
          errorThrow("Couldn't delete directory,\nit probably still contains files.\nProgram still stable.");
     }
}

char* filesysLoad(char* inPath) {
     if(filesysFileExists(inPath) == 0) {
          dlgFileError(inPath);
          return NULL;
     }
     FILE* tempFile = fopen(inPath, "rb");
     if(tempFile == NULL) {
          dlgFileError(inPath);
          return NULL;
     }
     fseek(tempFile, 0 , SEEK_END);
     long tempFileLen = ftell(tempFile);
     char* tempOut = (char*) malloc((tempFileLen + 1) * sizeof(char));
     rewind(tempFile);
     fread(&tempOut[0], 1, tempFileLen, tempFile);
     fclose(tempFile);
     tempOut[tempFileLen] = 0;
     return tempOut;
} 
