#include <psptypes.h>
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include "graphics.h"
#include "error.h"
#include "font.h"
#include "main.h"

Color fontColor = 0xFF000000;

void fontPrint(int inX, int inY, char* inString, Font* inFont) {
     if(inFont->fontFiller != fontColor) {
          imageReplaceColorIgnoreAlpha(inFont->fontFiller, fontColor, inFont->fontData);
          inFont->fontFiller = fontColor;
     }
     int i = 0; int oldInX = inX;
     while(inString[i] != 0) {
          if(inString[i] == 10)
               inY += inFont->fontHeight + 1;
          else if(inString[i] == 13)
               inX = oldInX;
          else
               imageDrawSectionAlpha((inString[i] & 0x0F) * inFont->fontWidth, ((inString[i] & 0xF0) >> 4) * inFont->fontHeight, inFont->fontWidth, inFont->fontHeight, inX, inY, inFont->fontData);
          inX += inFont->fontWidth + inFont->fontHSpace; i++;
     }
}

void fontPrintChar(int inX, int inY, unsigned char inChar, Font* inFont) {
     if(inFont->fontFiller != fontColor) {
          imageReplaceColorIgnoreAlpha(inFont->fontFiller, fontColor, inFont->fontData);
          inFont->fontFiller = fontColor;
     }
     imageDrawSectionAlpha((inChar & 0x0F) * inFont->fontWidth, ((inChar & 0xF0) >> 4) * inFont->fontHeight, inFont->fontWidth, inFont->fontHeight, inX, inY, inFont->fontData);
}

void fontPrintFromRight(int inX, int inY, char* inString, Font* inFont) {
     if(inFont->fontFiller != fontColor) {
          imageReplaceColorIgnoreAlpha(inFont->fontFiller, fontColor, inFont->fontData);
          inFont->fontFiller = fontColor;
     }
     int i = 0; int oldInX = inX;
     while(inString[i] != 0)
          i++;
     while(i != 0) {
          inX -= (inFont->fontWidth + inFont->fontHSpace); i--;
          if(inString[i] == 10)
               inY += (inFont->fontHeight + inFont->fontVSpace);
          else if(inString[i] == 13)
               inX = oldInX;
          else
               imageDrawSectionAlpha((inString[i] & 0x0F) * inFont->fontWidth, ((inString[i] & 0xF0) >> 4) * inFont->fontHeight, inFont->fontWidth, inFont->fontHeight, inX, inY, inFont->fontData);
     }
}

void fontPrintCenter(int inY, char* inString, Font* inFont) {
     fontPrint((240 - ((strlen(inString) * (inFont->fontWidth + inFont->fontHSpace)) >> 1)), inY, inString, inFont);
}

void fontPrintCenterMultiline(int inY, char* inString, Font* inFont) {
     char* tempLine = NULL;
     int tempPtr = 0;
     int tempLast = 0;
     int i = 0;
     int tempY = inY;
     while(inString[tempPtr] != 0) {
          if((inString[tempPtr] == ASCII_LF) && ((tempPtr - tempLast) > 0)) {
               if(tempLine != NULL)
                    free(tempLine);
               tempLine = (char*) malloc(((tempPtr - tempLast) + 1) * sizeof(char));
               for(i = tempLast; i < tempPtr; i++)
                    tempLine[i - tempLast] = inString[i];
               tempLine[tempPtr - tempLast] = 0;
               fontPrintCenter(tempY, tempLine, inFont);
               tempLast = tempPtr + 1;
               tempY += inFont->fontHeight + 1;
          }
          tempPtr++;
     }
     if(tempLine != NULL)
          free(tempLine);
     tempLine = (char*) malloc(((tempPtr - tempLast) + 1) * sizeof(char));
     for(i = tempLast; i < tempPtr; i++)
          tempLine[i - tempLast] = inString[i];
     tempLine[tempPtr - tempLast] = 0;
     fontPrintCenter(tempY, tempLine, inFont);
     if(tempLine != NULL)
          free(tempLine);
}

Font* fontLoad(char* inPath, int inHSpace, int inVSpace) {
      Font* tempFont = (Font*) malloc(sizeof(Font));
      if (!tempFont) {
           char tempError[256];
           sprintf(tempError, "Couldn't load font (%s).\nOut of memory.", inPath);
           errorThrow(tempError);
      }
      
      tempFont->fontData = imageLoad(inPath);
      tempFont->fontWidth = (tempFont->fontData->imageWidth >> 4);
      tempFont->fontHeight = (tempFont->fontData->imageHeight >> 4);
      tempFont->fontFiller = 0xFF000000;
      tempFont->fontHSpace = inHSpace;
      tempFont->fontVSpace = inVSpace;
      imageReplaceColor(FONT_ALPHA_COLOR, 0x00000000, tempFont->fontData);
      imageReplaceColor(FONT_ALPHA_UPPER_COLOR, 0x00000000, tempFont->fontData);
      imageReplaceColor(FONT_ALPHA_LOWER_COLOR, 0x00000000, tempFont->fontData);
      return tempFont;
}

void fontFree(Font* inFont) {
     imageFree(inFont->fontData);
     free(inFont);
}

void fontSetColor(Color inColor) {
     fontColor = inColor;
}

void fontInvertColor() {
     fontColor = colorInvert(fontColor);
}
