#ifndef GRAPHICS_H
#define GRAPHICS_H

#include <psptypes.h>

extern u32* g_vram_base;

#define	PSP_LINE_SIZE 512
#define SCREEN_WIDTH 480
#define SCREEN_HEIGHT 272

typedef u32 Color;
#define A(color) ((u8)(color >> 24 & 0xFF))
#define B(color) ((u8)(color >> 16 & 0xFF))
#define G(color) ((u8)(color >> 8 & 0xFF))
#define R(color) ((u8)(color & 0xFF))

typedef struct {
	int imageTextureWidth;
	int imageTextureHeight;
	int imageWidth;
	int imageHeight;
	Color* imageData;
} Image;

typedef struct {
	char* imageStackPath;
	Image* imageStackImage;
	void* imageStackNext;
} ImageStack_Entry;

typedef struct {
     int viewportX, viewportY;
     int viewportWidth, viewportHeight;
} Viewport;

extern void graphicsInit();
extern void graphicsDisable();
extern Color* graphicsVRamDrawBuffer();
extern Color* graphicsVRamDisplayBuffer();
extern void graphicsSwapBuffers();
extern void graphicsCopyBuffers();

extern Image* imageCreate(int inWidth, int inHeight);
extern Image* imageFromScreen();
extern Image* imageFromBuffer();
extern Image* imageThumbnail(Image* inImage);
extern Image* imageLoad(char* inPath);
extern void imageSave(const char* inPath, Image* inImage, char inSaveAlpha);
extern void imageFree(Image* inImage);
extern void imageClear(Color inColor, Image* inImage);
extern Color imageGetPixel(int inX, int inY, Image* inImage);
extern void imageSetPixel(int inX, int inY, Color inColor, Image* inImage);
extern void imageDrawLine(int inX0, int inY0, int inX1, int inY1, Color inColor, Image* inImage);
extern void imageDrawRect(int inX, int inY, int inWidth, int inHeight, Color inColor, Image* inImage);
extern void imageDrawFilledRect(int inX, int inY, int inWidth, int inHeight, Color inColor, Image* inImage);
extern void imageDraw(int inDrawX, int inDrawY, Image* inImage);
extern void imageDrawCenter(Image* inImage);
extern void imageDrawAlpha(int inDrawX, int inDrawY, Image* inImage);
extern void imageDrawSection(int inStartX, int inStartY, int inWidth, int inHeight, int inDrawX, int inDrawY, Image* inImage);
extern void imageDrawSectionAlpha(int inStartX, int inStartY, int inWidth, int inHeight, int inDrawX, int inDrawY, Image* inImage);
extern void imageDrawImageSection(int inStartX, int inStartY, int inWidth, int inHeight, int inDestX, int inDestY, Image* inSource, Image* inDestination);
extern void imageDrawImageSectionAlpha(int inStartX, int inStartY, int inWidth, int inHeight, int inDestX, int inDestY, Image* inSource, Image* inDestination);
extern void imagePrintText(int inX, int inY, const char* inText, Color inColor, Image* inImage);
extern void imageReplaceColor(Color inReplace, Color inColor, Image* inImage);
extern void imageReplaceColorIgnoreAlpha(Color inReplace, Color inColor, Image* inImage);

extern void screenClear(Color inColor);
extern Color screenGetPixel(int inX, int inY);
extern void screenSetPixel(int inX, int inY, Color inColor);
extern void screenDrawLine(int inX0, int inY0, int inX1, int inY1, Color inColor);
extern void screenDrawInvRect(int inX, int inY, int inWidth, int inHeight);
extern void screenDrawRect(int inX, int inY, int inWidth, int inHeight, Color inColor);
extern void screenDrawFilledRect(int inX, int inY, int inWidth, int inHeight, Color inColor);
extern void screenPrintText(int inX, int inY, const char* inText, Color inColor);
extern void screenDrawWindow(unsigned int inX, unsigned int inY, unsigned int inWidth, unsigned int inHeight);
extern void screenEffectWaves(int inHeight, int inStart);
extern void screenEffectFade(unsigned int inFade);

extern Color colorMerge(Color inColorA, Color inColorB);
extern Color colorBlend(Color inColorSrc, Color inColorDest);
extern Color colorFade(Color inColor, unsigned int inFade);
extern Color colorInvert(Color inColor);

extern ImageStack_Entry* imageStackAdd(char* inPath, Image* inImage);
extern ImageStack_Entry* imageStackFindByPath(char* inPath);
extern ImageStack_Entry* imageStackFindByImage(Image* inImage);
extern void imageStackRemove(Image* inImage);
extern void imageStackFree();

#endif
