#include <pspkernel.h>
#include <pspdebug.h>
#include <pspdisplay.h>
#include <pspctrl.h>
#include <psppower.h>
#include <psprtc.h>
#include <pspgu.h>
#include <png.h>
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include "graphics.h"
#include "keyboard.h"
#include "main.h"
#include "menu.h"
#include "error.h"
#include "textbox.h"
#include "dialog.h"
#include "input.h"
#include "filesys.h"
#include "fileBrowser.h"
#include "sprite.h"
#include "recentFiles.h"
#include "settings.h"

PSP_MODULE_INFO("FlatEditPSP", 0, 1, 1);

#define printf pspDebugScreenPrintf

Font* defaultFont = NULL;
TextBox* txtbMain = NULL;
Keyboard* keybMain = NULL;
Sprite* keyrefDPad = NULL;
Sprite* keyrefButtons = NULL;
Sprite* keyrefPadDesc = NULL;

/* Exit callback */
int exit_callback(int arg1, int arg2, void *common) {
	sceKernelExitGame();
	return 0;
}

/* Callback thread */
int CallbackThread(SceSize args, void *argp) {
	int cbid;

	cbid = sceKernelCreateCallback("Exit Callback", exit_callback, NULL);
	sceKernelRegisterExitCallback(cbid);

	sceKernelSleepThreadCB();

	return 0;
}

/* Sets up the callback thread and returns its thread id */
int SetupCallbacks(void) {
	int thid = 0;

	thid = sceKernelCreateThread("update_thread", CallbackThread, 0x11, 0xFA0, 0, 0);
	if(thid >= 0) {
		sceKernelStartThread(thid, 0, 0);
	}

	return thid;
}


int main() {
     pspDebugScreenInit();
     SetupCallbacks();

     printf("Loading settings...");
     if(filesysFileExists("settings/settings.cfg"))
          settingsLoad();
     else
          settingsSave();


     printf("Setting clock frequency to (222, 222, 111)...\n");
     scePowerSetClockFrequency(222, 222, 111);
     
     printf("Initializing graphics...\n");
     graphicsInit();
     
     printf("Initializing keyboard...\n");
     keybInit();
     keybMain = keybCreate(2, (SCREEN_HEIGHT - keybImgLo->imageHeight) - 2);
     
     printf("Initializing menu...\n");
     menuInit();

     char tempString[256];
     printf("Loading default font...\n");
     sprintf(tempString, "skins/%s/defaultFont.png", skinPath);
     defaultFont = fontLoad(tempString, 0, 0);

     printf("Creating textbox...\n");
     Font* tempFont;
     if(strcmp(fontPath, "defaultFont") == 0) {
          tempFont = defaultFont;
     } else {
          sprintf(tempString, "skins/%s/%s.png", skinPath, fontPath);
          tempFont = fontLoad(tempString, 0, 0);
     }
     if(keybEnabled == FALSE)
          txtbMain = txtbCreate(TEXTBOX_SIZE, tempFont, 0, 16, SCREEN_WIDTH, (SCREEN_HEIGHT - 24));
     else
          txtbMain = txtbCreate(TEXTBOX_SIZE, tempFont, 128, 16, (SCREEN_WIDTH - 136), (SCREEN_HEIGHT - 24));

     printf("Initializing input...\n");
     inputInit();
     
     printf("Initializing file browser...\n");
     fileBrowserInit();
     
     printf("Initializing recent files list...\n");
     recentFilesLoad();
     
     printf("Loading keyref images...\n");
     sprintf(tempString, "skins/%s/keyrefDPad.png", skinPath);
     keyrefDPad = spriteLoadAuto(tempString, 64, 64);
     sprintf(tempString, "skins/%s/keyrefButtons.png", skinPath);
     keyrefButtons = spriteLoadAuto(tempString, 22, 22);
     sprintf(tempString, "skins/%s/keyrefPadDesc.png", skinPath);
     keyrefPadDesc = spriteLoadAuto(tempString, 64, 64);
     
     
     printf("Starting main loop...\n");
     
     fontSetColor(0xFFFFFFFF);
     sprintf(tempString, "skins/%s/splash.png", skinPath);
     Image* pxpSplash = imageLoad(tempString);
     int i;
     for(i = 0; i < 150; i++) {
          inputPoll();
          if((inputRepeat[INPUT_TRIANGLE] != 0) || (inputRepeat[INPUT_SQUARE] != 0) || (inputRepeat[INPUT_CROSS] != 0) || (inputRepeat[INPUT_CIRCLE] != 0))
               break;
          sceDisplayWaitVblankStart();
          imageDraw(0, 0, pxpSplash);
          //fontPrint(2, (SCREEN_HEIGHT - 46), "FlatEditPSP version 1.5.0, Copyright (C) 2007 Ben Brewer", defaultFont);
          //fontPrint(2, (SCREEN_HEIGHT - 37), "FlatEditPSP comes with ABSOLUTELY NO WARRANTY; for details", defaultFont);
          //fontPrint(2, (SCREEN_HEIGHT - 28), "see Help->Copying.  This is free software, and you are welcome", defaultFont);
          //fontPrint(2, (SCREEN_HEIGHT - 19), "to redistribute it under certain conditions; select Help->Copying", defaultFont);
          //fontPrint(2, (SCREEN_HEIGHT - 10), "for details.", defaultFont);
          graphicsSwapBuffers();
     }
     imageFree(pxpSplash);
     fontSetColor(0xFF000000);
     
     stateLoad();
     
     while(1) {
          inputPoll();
          
          if((inputRepeat[INPUT_LTRIGGER] == 1) || (inputRepeat[INPUT_LTRIGGER] > INPUT_REPEATRATE)) {
               if(menuFocus != 0) {
                    keybMain->keybFocus = 0;
                    txtbMain->txtbFocus = 1;
                    menuFocus = 0;
               } else if(keybMain->keybFocus != 0) {
                    keybMain->keybFocus = 0;
                    txtbMain->txtbFocus = 0;
                    menuFocus = 1;
               } else if(txtbMain->txtbFocus != 0) {
                    txtbMain->txtbFocus = 0;
                    menuFocus = 0;
                    if(keybEnabled != FALSE)
                         keybMain->keybFocus = 1;
                    else
                         menuFocus = 1;
               } else {
                    if(keybEnabled != FALSE)
                        keybMain->keybFocus = 1;
                    else
                        txtbMain->txtbFocus = 1;
               }
          }
          
          if((inputRepeat[INPUT_RTRIGGER] == 1) || (inputRepeat[INPUT_RTRIGGER] > INPUT_REPEATRATE)) {
               if(menuFocus != 0) {
                    txtbMain->txtbFocus = 0;
                    menuFocus = 0;
                    if(keybEnabled != FALSE)
                         keybMain->keybFocus = 1;
                    else
                         txtbMain->txtbFocus = 1;
               } else if(keybMain->keybFocus != 0) {
                    keybMain->keybFocus = 0;
                    txtbMain->txtbFocus = 1;
                    menuFocus = 0;
               } else if(txtbMain->txtbFocus != 0) {
                    keybMain->keybFocus = 0;
                    txtbMain->txtbFocus = 0;
                    menuFocus = 1;
               } else {
                    if(keybEnabled != FALSE)
                        keybMain->keybFocus = 1;
                    else
                        txtbMain->txtbFocus = 1;
               }
          }
          
          keybTick(keybMain);
          menuTick();
          txtbAddFromKeyboard(txtbMain, keybMain);
          txtbTick(txtbMain);
                                            

          sceDisplayWaitVblankStart();
          renderLoop();
          graphicsSwapBuffers();
     }
	
     terminate();
     return 0;
}

void terminate() {
     //scePowerSetClockFrequency(222, 222, 111);
     stateSave();
     inputDeInit();
     imageStackFree();
     keybFree(keybMain);
     txtbFree(txtbMain);
     sceKernelExitGame();
}

void stateSave() {
     recentFilesSave();
     //settingsSave();
     
     if(stateEnabled == FALSE)
          return;

     char tempString[256];
     sceDisplayWaitVblankStart();
     graphicsCopyBuffers();
     screenDrawFilledRect((SCREEN_WIDTH >> 1) - 64, (SCREEN_HEIGHT >> 1) - 16, 128, 32, menuBackColor);
     screenDrawRect((SCREEN_WIDTH >> 1) - 64, (SCREEN_HEIGHT >> 1) - 16, 128, 32, menuBorderColor);
     fontSetColor(menuForeColor);
     sprintf(tempString, "Saving state.");
     fontPrintCenter((SCREEN_HEIGHT >> 1) - (defaultFont->fontHeight >> 1), tempString, defaultFont);
     graphicsSwapBuffers();
     
     
     
     sprintf(tempString, "settings/state.fes");
     FILE* tempFile = fopen(tempString, "wb");
     if(tempFile == NULL) {
          errorThrow("Couldn't save state.\nFile Error.");
          return;
     }
     int tempVer = (VERSION_MAJOR * 10000) + (VERSION_MINOR * 100) + VERSION_REVISION;
     fwrite(&tempVer, 1, sizeof(int), tempFile);
     fwrite(txtbMain, 1, sizeof(TextBox), tempFile);
     fwrite(&fileBrowserCWD[0], 128, sizeof(char), tempFile);
     fclose(tempFile);
     txtbSave(txtbMain, "settings/state.txt", 1);
}

void stateLoad() {
     if(stateEnabled == FALSE)
          return;
     if((filesysFileExists("settings/state.fes") <= 0) || (filesysFileExists("state.txt") <= 0))
          return;
     
     char tempString[256];
     sceDisplayWaitVblankStart();
     graphicsCopyBuffers();
     screenDrawFilledRect((SCREEN_WIDTH >> 1) - 64, (SCREEN_HEIGHT >> 1) - 16, 128, 32, menuBackColor);
     screenDrawRect((SCREEN_WIDTH >> 1) - 64, (SCREEN_HEIGHT >> 1) - 16, 128, 32, menuBorderColor);
     fontSetColor(menuForeColor);
     sprintf(tempString, "Loading state.");
     fontPrintCenter((SCREEN_HEIGHT >> 1) - (defaultFont->fontHeight >> 1), tempString, defaultFont);
     graphicsSwapBuffers();
     
     TextBox* tempTextBox = (TextBox*) malloc(sizeof(TextBox));
     if(tempTextBox == NULL) {
          errorThrow("Couldn't create a text box structure.\nOut of memory.");
          return;
     }
     sprintf(tempString, "settings/state.fes");
     FILE* tempFile = fopen(tempString, "rb");
     if(tempFile == NULL) {
          errorThrow("Couldn't load state.\nFile Error.\nProgram stable.");
          free(tempTextBox);
          fclose(tempFile);
          return;
     }
     int tempVer = 0;
     fread(&tempVer, 1, sizeof(int), tempFile);
     if(tempVer != ((VERSION_MAJOR * 10000) + (VERSION_MINOR * 100) + VERSION_REVISION)) {
          errorThrow("Incompatible saved state.\nLoading as normal\nProgram stable.");
          free(tempTextBox);
          fclose(tempFile);
          return;
     }
     fread(tempTextBox, 1, sizeof(TextBox), tempFile);
     fread(&fileBrowserCWD[0], 128, sizeof(char), tempFile);
     fclose(tempFile);
     txtbMain->txtbSize = tempTextBox->txtbSize;
     free(txtbMain->txtbText);
     txtbMain->txtbText = (unsigned char*) malloc(txtbMain->txtbSize * sizeof(char));
     txtbOpen(txtbMain, "settings/state.txt", 1);
     txtbMain->txtbSelStart = tempTextBox->txtbSelStart;
     txtbMain->txtbSelEnd = tempTextBox->txtbSelEnd;
     strcpy(txtbMain->txtbPath, tempTextBox->txtbPath);
     txtbMain->txtbChanged = tempTextBox->txtbChanged;
     txtbMain->txtbOvrMode = tempTextBox->txtbOvrMode;
     txtbMain->txtbScrollH = tempTextBox->txtbScrollH;
     txtbMain->txtbScrollV = tempTextBox->txtbScrollV;
     
     free(tempTextBox);
}

void renderLoop() {
     screenClear(0xFFFFFFFF);
     txtbDraw(txtbMain);
     menuDraw();
     if(keybEnabled != FALSE)
          keybDraw(keybMain);

     //Draw keyref images
     int tempFrame = 0;
     
     if(inputRepeat[INPUT_UP] != 0)
          tempFrame = 1;
     else if(inputRepeat[INPUT_DOWN] != 0)
          tempFrame = 2;
     else if(inputRepeat[INPUT_LEFT] != 0)
          tempFrame = 3;
     else if(inputRepeat[INPUT_RIGHT] != 0)
          tempFrame = 4;

     if(keybEnabled == FALSE)
          return;

     fontSetColor(menuForeColor);

     char tempString[256];
     sprintf(tempString, "Row: %i/%i", (txtbMain->txtbSelRow + 1), (txtbMain->txtbLineCount - 1) + 1);
     fontPrint(1, 17, tempString, defaultFont);

     sprintf(tempString, "Col: %i/%i", txtbMain->txtbSelCol, txtbLineWidth(txtbMain, txtbMain->txtbSelRow));
     fontPrint(1, 29, tempString, defaultFont);
     
     if(txtbMain->txtbOvrMode == 0)
          sprintf(tempString, "Mode: INSERT");
     else
          sprintf(tempString, "Mode: OVERWRITE");
     fontPrint(1, 41, tempString, defaultFont);
     
     int tempInt = scePowerGetBatteryLifePercent();
     sprintf(tempString, "Batt: %i%%", tempInt);
     if(scePowerIsPowerOnline() != 0)
          sprintf(tempString, "%s (AC)", tempString);
     if(scePowerIsLowBattery() != 0) {
          fontSetColor(0xFF0000FF);
          fontPrint(1, 76, tempString, defaultFont);
          fontSetColor(0xFF000000);
     } else {
          fontPrint(1, 76, tempString, defaultFont);
     }
     
     pspTime tempTime;
     sceRtcGetCurrentClockLocalTime(&tempTime);
     if(tempTime.microseconds < 500000)
          sprintf(tempString, "Time: %02i:%02i:%02i", tempTime.hour, tempTime.minutes, tempTime.seconds);
     else
          sprintf(tempString, "Time: %02i:%02i %02i", tempTime.hour, tempTime.minutes, tempTime.seconds);
     fontPrint(1, 88, tempString, defaultFont);

     if(menuFocus == 0)
          spriteDrawFrame(keyrefDPad, 31, (SCREEN_HEIGHT - 141), tempFrame);
     else
          spriteDrawFrame(keyrefDPad, 31, (SCREEN_HEIGHT - 141), 0);
          
     tempFrame = 5;
     if(inputRepeat[INPUT_CROSS] != 0) {
          tempFrame = 3;
          if(menuFocus == 0)
               spriteDrawFrame(keyrefButtons, 72, (SCREEN_HEIGHT - 100), 3);
     }
     if(inputRepeat[INPUT_TRIANGLE] != 0) {
          tempFrame = 1;
          if(menuFocus == 0)
               spriteDrawFrame(keyrefButtons, 32, (SCREEN_HEIGHT - 140), 1);
     }
     if(inputRepeat[INPUT_CIRCLE] != 0) {
          tempFrame = 2;
          if(menuFocus == 0)
               spriteDrawFrame(keyrefButtons, 72, (SCREEN_HEIGHT - 140), 2);
     }
     if(inputRepeat[INPUT_SQUARE] != 0) {
          tempFrame = 0;
          if(menuFocus == 0)
               spriteDrawFrame(keyrefButtons, 32, (SCREEN_HEIGHT - 100), 0);
     }
     if(keybMain->keybFocus != 0) {
          tempFrame = 4;
     } else if((txtbMain->txtbFocus == 0) && (keybMain->keybFocus == 0)) {
          tempFrame = 5;
     }
          
     spriteDrawFrame(keyrefPadDesc, 31, (SCREEN_HEIGHT - 141), tempFrame);
}
