#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <psptypes.h>
#include "settings.h"
#include "graphics.h"
#include "main.h"
#include "error.h"
#include "filesys.h"

Color txtbForeColor   = 0xFFFFFFFF;//0xFF000000;
Color txtbBackColor   = 0xFF000000;//0xFFFFFFFF;
Color txtbSelColor    = 0xFFBF0000;//0xFF800000;
Color menuForeColor   = 0xFFBFBFBF;//0xFF000000;
Color menuBackColor   = 0xFF3F3F3F;//0xFFC0C0C0;
Color menuBorderColor = 0xFFFFFFFF;//0xFF000000;
char skinPath[64]     = "default";
char fontPath[64]     = "defaultFont";

char txtbNewLineType  = NEWLINE_IGNORE;
char stateEnabled     = TRUE;
char keybEnabled      = TRUE;

u32 hexStrToInt(char* inString) {
     if((inString == NULL) || (inString[0] == 0))
          return 0;
     u32 tempPtr = 0;
     u8 tempChar = 0;
     u32 tempOut = 0;
     while(inString[tempPtr] != 0) {
          tempOut *= 16;
          tempChar = inString[tempPtr];
          if((tempChar <= 57) && (tempChar > 48))
               tempOut += (tempChar - 48);
          if(tempChar >= 97)
               tempChar -= 32;
          if((tempChar >= 65) && (tempChar <= 70))
               tempOut += (tempChar - 55);
          tempPtr++;
     }
     return tempOut;
}

u32 numStrToInt(char* inString) {
     if((inString == NULL) || (inString[0] == 0))
          return 0;
     u32 tempPtr = 0;
     u8 tempChar = 0;
     u32 tempOut = 0;
     while(inString[tempPtr] != 0) {
          tempOut *= 10;
          tempChar = inString[tempPtr];
          if((tempChar <= 57) && (tempChar > 48))
               tempOut += (tempChar - 48);
          tempPtr++;
     }
     return tempOut;
}

void settingsDecode(char* inString) {
     if((inString == NULL) || (inString[0] == 0) || (inString[0] == 35))
          return;
     char tempName[256];
     char tempVal[256];
     tempName[0] = 0;
     tempVal[0] = 0;
     int tempPtr = 0;
     int tempStrPtr = 0;
     while(inString[tempPtr] != 61) {
          if((inString[tempPtr] != ASCII_SPACE) && (tempStrPtr <= 255) && (inString[tempPtr] != ASCII_CR)){
               tempName[tempStrPtr] = inString[tempPtr];
               tempStrPtr++;
          }
          tempPtr++;
     }
     tempName[tempStrPtr] = 0;
     tempPtr++;
     tempStrPtr = 0;
     while((inString[tempPtr] != ASCII_LF) && (inString[tempPtr] != 0)) {
          if(inString[tempPtr] != ASCII_SPACE) {
               tempVal[tempStrPtr] = inString[tempPtr];
               tempStrPtr++;
          }
          tempPtr++;
     }
     tempVal[tempStrPtr] = 0;
     if(strcmp(tempName, "txtbForeColor") == 0)
          txtbForeColor = hexStrToInt(tempVal);
     if(strcmp(tempName, "txtbBackColor") == 0)
          txtbBackColor = hexStrToInt(tempVal);
     if(strcmp(tempName, "txtbSelColor") == 0)
          txtbSelColor = hexStrToInt(tempVal);
     if(strcmp(tempName, "menuForeColor") == 0)
          menuForeColor = hexStrToInt(tempVal);
     if(strcmp(tempName, "menuBackColor") == 0)
          menuBackColor = hexStrToInt(tempVal);
     if(strcmp(tempName, "menuBorderColor") == 0)
          menuBorderColor = hexStrToInt(tempVal);
     if(strcmp(tempName, "skinPath") == 0)
          strcpy(skinPath, tempVal);
     if(strcmp(tempName, "txtbNewLineType") == 0)
          txtbNewLineType = numStrToInt(tempVal);
     if(strcmp(tempName, "stateEnabled") == 0)
          stateEnabled = numStrToInt(tempVal);
     if(strcmp(tempName, "keybEnabled") == 0)
          keybEnabled = numStrToInt(tempVal);
     if(strcmp(tempName, "fontPath") == 0)
          strcpy(fontPath, tempVal);
}

void settingsSave() {
     if(filesysIsFile("./settings") != 0)
          filesysMkdir("./settings");
     char tempString[512];
     sprintf(tempString, "settings/settings.cfg");
     FILE* tempFile = fopen(tempString, "wb");
     if(tempFile == NULL) {
           errorThrow("Couldn't open file 'settings/settings.cfg\nFor writing.\nProgram stable.");
           return;
     }
     sprintf(tempString, "txtbForeColor=%08X\n", txtbForeColor);
     fwrite(tempString, 1, strlen(tempString), tempFile);
     sprintf(tempString, "txtbBackColor=%08X\n", txtbBackColor);
     fwrite(tempString, 1, strlen(tempString), tempFile);
     sprintf(tempString, "txtbSelColor=%08X\n", txtbSelColor);
     fwrite(tempString, 1, strlen(tempString), tempFile);
     sprintf(tempString, "menuForeColor=%08X\n", menuForeColor);
     fwrite(tempString, 1, strlen(tempString), tempFile);
     sprintf(tempString, "menuBackColor=%08X\n", menuBackColor);
     fwrite(tempString, 1, strlen(tempString), tempFile);
     sprintf(tempString, "menuBorderColor=%08X\n", menuBorderColor);
     fwrite(tempString, 1, strlen(tempString), tempFile);
     sprintf(tempString, "skinPath=%s\n", skinPath);
     fwrite(tempString, 1, strlen(tempString), tempFile);
     sprintf(tempString, "txtbNewLineType=%i\n", txtbNewLineType);
     fwrite(tempString, 1, strlen(tempString), tempFile);
     sprintf(tempString, "stateEnabled=%i\n", stateEnabled);
     fwrite(tempString, 1, strlen(tempString), tempFile);
     sprintf(tempString, "keybEnabled=%i\n", keybEnabled);
     fwrite(tempString, 1, strlen(tempString), tempFile);
     sprintf(tempString, "fontPath=%s\n", fontPath);
     fwrite(tempString, 1, strlen(tempString), tempFile);
     fclose(tempFile);
}

void settingsLoad() {
     char tempString[512];
     if(filesysFileExists("settings/settings.cfg") <= 0)
           return;
     sprintf(tempString, "settings/settings.cfg");
     FILE* tempFile = fopen(tempString, "rb");
     if(tempFile == NULL) {
           errorThrow("Couldn't open file 'settings/settings.cfg\nFor reading.\nProgram stable.");
           return;
     }
     while(!feof(tempFile) && !ferror(tempFile)) {
          fgets(tempString, 512, tempFile);
          settingsDecode(tempString);
     }
     fclose(tempFile);
}
