#include <psptypes.h>
#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include "textbox.h"
#include "graphics.h"
#include "font.h"
#include "error.h"
#include "main.h"
#include "keyboard.h"
#include "filesys.h"
#include "dialog.h"
#include "input.h"
#include "copyBuffer.h"
#include "settings.h"

TextBox* txtbCreate(unsigned int inSize, Font* inFont, int inLeft, int inTop, unsigned int inWidth, unsigned int inHeight) {
     TextBox* tempTextBox = (TextBox*) malloc(sizeof(TextBox));
     if(tempTextBox == NULL) {
          errorThrow("Couldn't create textbox.\nOut of memory.");
          return NULL;
     }
     tempTextBox->txtbText = (unsigned char*) malloc(inSize * sizeof(unsigned char));
     while((tempTextBox->txtbText == NULL) && (inSize > 8192)) {
          inSize = (inSize >> 1);
          tempTextBox->txtbText = (unsigned char*) malloc(inSize * sizeof(unsigned char));
          errorThrow("Low on memory, halving maximum textbox size.");
     }
     if(tempTextBox->txtbText == NULL) {
          errorThrow("Couldn't create textbox.\nOut of memory.");
          free(tempTextBox);
          return NULL;
     }
     tempTextBox->txtbText[0] = 0;
     tempTextBox->txtbLeft = inLeft;
     tempTextBox->txtbTop = inTop;
     tempTextBox->txtbWidth = inWidth;
     tempTextBox->txtbHeight = inHeight;
     
     tempTextBox->txtbLineCount = 1;
     tempTextBox->txtbLines = (int*) memalign(32, tempTextBox->txtbLineCount * sizeof(int));
     if(tempTextBox->txtbLines == NULL) {
          errorThrow("Couldn't create textbox line cache.\nOut of memory.");
          free(tempTextBox->txtbText);
          free(tempTextBox);
          return NULL;
     }
     tempTextBox->txtbLines[0] = -1;
     tempTextBox->txtbLineMaxWidth = 0;
     
     tempTextBox->txtbScrollH = 0;
     tempTextBox->txtbScrollV = 0;
          
     tempTextBox->txtbSelStart = 0;
     tempTextBox->txtbSelEnd = 0;

     tempTextBox->txtbSelCol = 0;
     tempTextBox->txtbSelRow = 0;

     tempTextBox->txtbSize = inSize;
     tempTextBox->txtbPath[0] = 0;
     
     tempTextBox->txtbChanged = FALSE;
     tempTextBox->txtbAnim = 0;
     tempTextBox->txtbFocus = FALSE;

     tempTextBox->txtbFont = inFont;
     
     tempTextBox->txtbSelMode = 0;
     tempTextBox->txtbOvrMode = 0;
     return tempTextBox;
}

void txtbClear(TextBox* inTextBox) {
     inTextBox->txtbText[0] = 0;
     inTextBox->txtbSelStart = 0;
     inTextBox->txtbSelEnd = 0;
     inTextBox->txtbSelCol = 0;
     inTextBox->txtbSelRow = 0;
     inTextBox->txtbChanged = FALSE;
     inTextBox->txtbLineCount = 1;
     free(inTextBox->txtbLines);
     inTextBox->txtbLines = (int*) memalign(32, inTextBox->txtbLineCount * sizeof(int));
     if(inTextBox->txtbLines == NULL) {
          errorThrow("Couldn't create textbox line cache.\nOut of memory.");
     } else {
          inTextBox->txtbLines[0] = -1;
          inTextBox->txtbLineMaxWidth = 0;
     }
     inTextBox->txtbScrollH = 0;
     inTextBox->txtbScrollV = 0;
     inTextBox->txtbPath[0] = 0;
     inTextBox->txtbSelMode = 0;
     inTextBox->txtbOvrMode = 0;
}

void txtbTick(TextBox* inTextBox) {
     while(inTextBox->txtbSelRow < inTextBox->txtbScrollV)
          inTextBox->txtbScrollV--;
     while((inTextBox->txtbSelRow - inTextBox->txtbScrollV) >= ((inTextBox->txtbHeight - 2) / (inTextBox->txtbFont->fontHeight + 1)))
          inTextBox->txtbScrollV++;
     while(inTextBox->txtbSelCol < inTextBox->txtbScrollH)
          inTextBox->txtbScrollH--;
     while((inTextBox->txtbSelCol - inTextBox->txtbScrollH) >= ((inTextBox->txtbWidth - 2) / (inTextBox->txtbFont->fontWidth + 1)))
          inTextBox->txtbScrollH++;
          
     if(inTextBox->txtbFocus == FALSE)
          return;

     inTextBox->txtbAnim++;
     
     if(inputRepeat[INPUT_SQUARE] != 0) {
          if((inputRepeat[INPUT_LEFT] == 1) || (inputRepeat[INPUT_LEFT] > INPUT_REPEATRATE)) {
               txtbHome(inTextBox);
          }
          if((inputRepeat[INPUT_RIGHT] == 1) || (inputRepeat[INPUT_RIGHT] > INPUT_REPEATRATE)) {
               txtbEnd(inTextBox);
          }
          if((inputRepeat[INPUT_UP] == 1) || (inputRepeat[INPUT_UP] > INPUT_REPEATRATE)) {
               txtbPageUp(inTextBox);
          }
          if((inputRepeat[INPUT_DOWN] == 1) || (inputRepeat[INPUT_DOWN] > INPUT_REPEATRATE)) {
               txtbPageDown(inTextBox);
          }
     } else if(inputRepeat[INPUT_CIRCLE] != 0) {
          if((inputRepeat[INPUT_LEFT] == 1) || (inputRepeat[INPUT_LEFT] > INPUT_REPEATRATE)) {
               txtbCut(inTextBox);
          }
          if((inputRepeat[INPUT_RIGHT] == 1) || (inputRepeat[INPUT_RIGHT] > INPUT_REPEATRATE)) {
               txtbPaste(inTextBox);
          }
          if((inputRepeat[INPUT_UP] == 1) || (inputRepeat[INPUT_UP] > INPUT_REPEATRATE)) {
               txtbCopy(inTextBox);
          }
          if((inputRepeat[INPUT_DOWN] == 1) || (inputRepeat[INPUT_DOWN] > INPUT_REPEATRATE)) {
               txtbDelete(inTextBox);
          }
     } else if(inputRepeat[INPUT_TRIANGLE] != 0) {
          if((inputRepeat[INPUT_LEFT] == 1) || (inputRepeat[INPUT_LEFT] > INPUT_REPEATRATE)) {
               txtbBackSpace(inTextBox);
          }
          if((inputRepeat[INPUT_RIGHT] == 1) || (inputRepeat[INPUT_RIGHT] > INPUT_REPEATRATE)) {
               txtbInsertChar(inTextBox, ASCII_SPACE);
          }
          if((inputRepeat[INPUT_UP] == 1) || (inputRepeat[INPUT_UP] > INPUT_REPEATRATE)) {
               inTextBox->txtbOvrMode ^= 1;
          }
          if((inputRepeat[INPUT_DOWN] == 1) || (inputRepeat[INPUT_DOWN] > INPUT_REPEATRATE)) {
               txtbInsertChar(inTextBox, ASCII_LF);
          }
     } else {
          if((inputRepeat[INPUT_LEFT] == 1) || (inputRepeat[INPUT_LEFT] > INPUT_REPEATRATE)) {
               txtbCursorMoveLeft(inTextBox);
          }
          if((inputRepeat[INPUT_RIGHT] == 1) || (inputRepeat[INPUT_RIGHT] > INPUT_REPEATRATE)) {
               txtbCursorMoveRight(inTextBox);
          }
          if((inputRepeat[INPUT_UP] == 1) || (inputRepeat[INPUT_UP] > INPUT_REPEATRATE)) {
               txtbCursorMoveUp(inTextBox);
          }
          if((inputRepeat[INPUT_DOWN] == 1) || (inputRepeat[INPUT_DOWN] > INPUT_REPEATRATE)) {
               txtbCursorMoveDown(inTextBox);
          }
     }
     if(inputRepeat[INPUT_CROSS] != 0)
          inTextBox->txtbSelMode = 1;
     else
          inTextBox->txtbSelMode = 0;
}

void txtbRecacheLines(TextBox* inTextBox) {
     inTextBox->txtbLineCount = 1;
     int tempPtr = 0;
     while(inTextBox->txtbText[tempPtr] != 0) {
          if(inTextBox->txtbText[tempPtr] == ASCII_LF)
               inTextBox->txtbLineCount++;
          tempPtr++;
     }
     if(inTextBox->txtbLines != NULL)
          free(inTextBox->txtbLines);
     inTextBox->txtbLines = (int*) memalign(32, inTextBox->txtbLineCount * sizeof(int));
     if(inTextBox->txtbLines == NULL) {
          errorThrow("Couldn't create textbox line cache.\nOut of memory.");
          return;
     }
     inTextBox->txtbLines[0] = -1;
     tempPtr = 0;
     int tempLinePtr = 1;
     inTextBox->txtbLineMaxWidth = 0;
     while(inTextBox->txtbText[tempPtr] != 0) {
          if(inTextBox->txtbText[tempPtr] == ASCII_LF) {
               inTextBox->txtbLines[tempLinePtr] = tempPtr;
               if(tempLinePtr > 0) {
                    if((tempPtr - inTextBox->txtbLines[tempLinePtr - 1]) > inTextBox->txtbLineMaxWidth) {
                         inTextBox->txtbLineMaxWidth = tempPtr - inTextBox->txtbLines[tempLinePtr - 1];
                    }
               } else {
                    inTextBox->txtbLineMaxWidth = tempPtr;
               }
               tempLinePtr++;
          }
          tempPtr++;
     }
     if(inTextBox->txtbLineCount <= 1) {
          inTextBox->txtbLineMaxWidth = strlen((char*)inTextBox->txtbText);
     } else {
          if((strlen((char*)inTextBox->txtbText) - inTextBox->txtbLines[inTextBox->txtbLineCount - 1]) > inTextBox->txtbLineMaxWidth)
               inTextBox->txtbLineMaxWidth = strlen((char*)inTextBox->txtbText) - inTextBox->txtbLines[inTextBox->txtbLineCount - 1];
     }
}

int txtbLineWidth(TextBox* inTextBox, int inLine) {
     if((inLine < 0) || (inLine >= inTextBox->txtbLineCount))
          return - 1;
     
     if(inLine == (inTextBox->txtbLineCount - 1)) {
          if((inLine == 0) && (inTextBox->txtbLineCount == 1)) {
               return strlen((char*)inTextBox->txtbText);
          } else {
               return strlen((char*)inTextBox->txtbText) - inTextBox->txtbLines[inTextBox->txtbLineCount - 1] - 1;
          }
     }
     return (inTextBox->txtbLines[inLine + 1] - inTextBox->txtbLines[inLine] - 1);
}

void txtbDraw(TextBox* inTextBox) {
     screenDrawFilledRect(inTextBox->txtbLeft, inTextBox->txtbTop, inTextBox->txtbWidth, inTextBox->txtbHeight, txtbBackColor);
     fontSetColor(txtbForeColor);

     int tempX = inTextBox->txtbLeft + 1 - (inTextBox->txtbScrollH * (inTextBox->txtbFont->fontWidth + 1));
     int tempY = inTextBox->txtbTop + 1;
     int tempPtr = inTextBox->txtbLines[inTextBox->txtbScrollV] + 1;
     int tempSelX = 0;
     int tempSelY = 0;
     if((tempPtr == inTextBox->txtbSelStart) && (tempX < (inTextBox->txtbLeft + inTextBox->txtbWidth)) && ((tempX + inTextBox->txtbFont->fontWidth) > inTextBox->txtbLeft)) {
          tempSelX = tempX;
          tempSelY = tempY;
     }
     while((tempY < (inTextBox->txtbTop + inTextBox->txtbHeight)) && (inTextBox->txtbText[tempPtr] != 0)) {
          if(inTextBox->txtbText[tempPtr] == ASCII_LF) {
               tempX = inTextBox->txtbLeft + 1 - (inTextBox->txtbScrollH * (inTextBox->txtbFont->fontWidth + inTextBox->txtbFont->fontHSpace));
               tempY += inTextBox->txtbFont->fontHeight + inTextBox->txtbFont->fontVSpace;
          } else if(inTextBox->txtbText[tempPtr] == ASCII_TAB) {
               int tempInt = inTextBox->txtbLeft + 1;
               while(tempX >= tempInt)
                    tempInt += (inTextBox->txtbFont->fontWidth * TEXTBOX_TABSPACING) + TEXTBOX_TABSPACING;
               if((inTextBox->txtbSelEnd > inTextBox->txtbSelStart) && (tempPtr >= inTextBox->txtbSelStart) && (tempPtr < inTextBox->txtbSelEnd)) {
                    screenDrawFilledRect(tempX, tempY, (tempInt - tempX), (inTextBox->txtbFont->fontHeight + inTextBox->txtbFont->fontVSpace), txtbSelColor);
               } else if((inTextBox->txtbSelEnd < inTextBox->txtbSelStart) && (tempPtr < inTextBox->txtbSelStart) && (tempPtr >= inTextBox->txtbSelEnd)) {
                    screenDrawFilledRect(tempX, tempY, (tempInt - tempX), (inTextBox->txtbFont->fontHeight + inTextBox->txtbFont->fontVSpace), txtbSelColor);
               }
               tempX = tempInt;
          } else if((inTextBox->txtbText[tempPtr] > 31) && (inTextBox->txtbText[tempPtr] != 127)) {
               if((tempX < (inTextBox->txtbLeft + inTextBox->txtbWidth)) && ((tempX + inTextBox->txtbFont->fontWidth) >= inTextBox->txtbLeft)) {
                    if(inTextBox->txtbSelStart != inTextBox->txtbSelEnd) {
                         if((inTextBox->txtbSelEnd > inTextBox->txtbSelStart) && (tempPtr >= inTextBox->txtbSelStart) && (tempPtr < inTextBox->txtbSelEnd)) {
                              fontInvertColor();
                              screenDrawFilledRect(tempX, tempY, (inTextBox->txtbFont->fontWidth + inTextBox->txtbFont->fontHSpace), (inTextBox->txtbFont->fontHeight + inTextBox->txtbFont->fontVSpace), txtbSelColor);
                              fontPrintChar(tempX, tempY, inTextBox->txtbText[tempPtr], inTextBox->txtbFont);
                              fontInvertColor();
                         } else if((inTextBox->txtbSelEnd < inTextBox->txtbSelStart) && (tempPtr < inTextBox->txtbSelStart) && (tempPtr >= inTextBox->txtbSelEnd)) {
                              fontInvertColor();
                              screenDrawFilledRect(tempX, tempY, (inTextBox->txtbFont->fontWidth + inTextBox->txtbFont->fontHSpace), (inTextBox->txtbFont->fontHeight + inTextBox->txtbFont->fontVSpace), txtbSelColor);
                              fontPrintChar(tempX, tempY, inTextBox->txtbText[tempPtr], inTextBox->txtbFont);
                              fontInvertColor();
                         } else {
                              fontPrintChar(tempX, tempY, inTextBox->txtbText[tempPtr], inTextBox->txtbFont);
                         }
                     } else {
                         fontPrintChar(tempX, tempY, inTextBox->txtbText[tempPtr], inTextBox->txtbFont);
                     }
               }
               tempX += inTextBox->txtbFont->fontWidth + inTextBox->txtbFont->fontHSpace;
          }
          tempPtr++;
          if((tempPtr == inTextBox->txtbSelStart) && (tempX < (inTextBox->txtbLeft + inTextBox->txtbWidth)) && ((tempX + inTextBox->txtbFont->fontWidth) > inTextBox->txtbLeft)) {
               tempSelX = tempX;
               tempSelY = tempY;
          }
     }
     if(((inTextBox->txtbAnim & 0x04) != 0) || (inTextBox->txtbFocus == 0)) {
          if(inTextBox->txtbOvrMode == 0) {
               screenDrawInvRect(tempSelX, tempSelY, 1,(inTextBox->txtbFont->fontHeight + inTextBox->txtbFont->fontHSpace));
          } else {
               screenDrawInvRect(tempSelX, tempSelY, (inTextBox->txtbFont->fontWidth + inTextBox->txtbFont->fontHSpace),(inTextBox->txtbFont->fontHeight + inTextBox->txtbFont->fontVSpace));
          }
     }
}

void txtbAddFromKeyboard(TextBox* inTextBox, Keyboard* inKeyboard) {
     unsigned char tempChar = keybBufferPop(inKeyboard);
     while(tempChar != 0) {
          if(tempChar == ASCII_BS) {
               txtbBackSpace(inTextBox);
          } else {
               txtbInsertChar(inTextBox, tempChar);
          }
          tempChar = keybBufferPop(inKeyboard);
     }
}

void txtbOpen(TextBox* inTextBox, char* inPath, char inQuiet) {
     if((inTextBox == NULL) || (inTextBox->txtbText == NULL))
          return;
     if(filesysFileExists(inPath) == 0) {
          dlgFileError(inPath);
          return;
     }
     FILE* tempFile = fopen(inPath, "rb");
     if(tempFile == NULL) {
          dlgFileError(inPath);
          return;
     }
     txtbClear(inTextBox);
     fseek(tempFile, 0 , SEEK_END);
     long tempFileLen = ftell(tempFile);
     if(tempFileLen > inTextBox->txtbSize) {
          inTextBox->txtbSize *= 1.5;
          free(inTextBox->txtbText);
          inTextBox->txtbText = (unsigned char*) malloc(inTextBox->txtbSize * sizeof(unsigned char));
          if(inTextBox->txtbText == NULL) {
               char tempString[256];
               sprintf(tempString, "Couldn't open file.\nFile too big ( %lu Mb ).\nOut of memory.", (tempFileLen >> 20));
               errorThrow(tempString);
               return;
          }
     }
     rewind(tempFile);
     fread(&inTextBox->txtbText[0], 1, tempFileLen, tempFile);
     fclose(tempFile);
     inTextBox->txtbText[tempFileLen] = 0;
     sprintf(inTextBox->txtbPath, "%s", inPath);
     txtbRecacheLines(inTextBox);
     
     char tempString[256];
     sprintf(tempString, "File opened.");
     if(inQuiet == 0)
          dlgMessage(tempString);
}

void txtbSave(TextBox* inTextBox, char* inPath, char inQuiet) {
     if((inTextBox == NULL) || (inTextBox->txtbText == NULL))
          return;
     FILE* tempFile = fopen(inPath, "wb");
     if(tempFile == NULL) {
          dlgFileError(inPath);
          return;
     }
     int tempPtr = 0;
     while(inTextBox->txtbText[tempPtr] != 0)
          tempPtr++;
     fwrite(&inTextBox->txtbText[0], 1, (tempPtr + 1), tempFile);
     fclose(tempFile);
     sprintf(inTextBox->txtbPath, "%s", inPath);
     inTextBox->txtbChanged = FALSE;
     
     char tempString[256];
     sprintf(tempString, "File saved.");
     if(inQuiet == 0)
          dlgMessage(tempString);
}

void txtbCursorMoveUp(TextBox* inTextBox) {
     if(inTextBox->txtbSelRow <= 0)
          return;
     int tempCol = inTextBox->txtbSelCol;
     inTextBox->txtbSelStart--;
     while((inTextBox->txtbText[inTextBox->txtbSelStart] != ASCII_LF) && (inTextBox->txtbSelStart > 0)) {
          inTextBox->txtbSelStart--;
     }
     txtbCursorCol(inTextBox);
     while(inTextBox->txtbSelCol > tempCol) {
          inTextBox->txtbSelStart--;
          txtbCursorCol(inTextBox);
     }
     if(inTextBox->txtbSelMode == 0)
          inTextBox->txtbSelEnd = inTextBox->txtbSelStart;
     txtbCursorRow(inTextBox);
}

void txtbCursorMoveDown(TextBox* inTextBox) {
     int tempCol = inTextBox->txtbSelCol;
     txtbEnd(inTextBox);
     txtbCursorMoveRight(inTextBox);
     txtbEnd(inTextBox);
     while(inTextBox->txtbSelCol > tempCol) {
          inTextBox->txtbSelStart--;
          txtbCursorCol(inTextBox);
     }
     if(inTextBox->txtbSelMode == 0)
          inTextBox->txtbSelEnd = inTextBox->txtbSelStart;
     txtbCursorRow(inTextBox);
}

void txtbCursorMoveLeft(TextBox* inTextBox) {
     if(inTextBox->txtbSelStart > 0)
          inTextBox->txtbSelStart--;
     if(inTextBox->txtbSelMode == 0)
          inTextBox->txtbSelEnd = inTextBox->txtbSelStart;
     txtbCursorRow(inTextBox); txtbCursorCol(inTextBox);
}

void txtbCursorMoveRight(TextBox* inTextBox) {
     if(inTextBox->txtbSelStart >= inTextBox->txtbSize)
          return;
     if(inTextBox->txtbText[inTextBox->txtbSelStart] != 0)
          inTextBox->txtbSelStart++;
     if(inTextBox->txtbSelMode == 0)
          inTextBox->txtbSelEnd = inTextBox->txtbSelStart;
     txtbCursorRow(inTextBox); txtbCursorCol(inTextBox);
}

void txtbCursorCol(TextBox* inTextBox) {
     int tempLinePtr = (inTextBox->txtbLineCount - 1);
     while((inTextBox->txtbLines[tempLinePtr] >= inTextBox->txtbSelStart) && (tempLinePtr > 0))
          tempLinePtr--;
     inTextBox->txtbSelCol = (inTextBox->txtbSelStart - inTextBox->txtbLines[tempLinePtr] - 1);
}

void txtbCursorRow(TextBox* inTextBox) {
     int tempLinePtr = (inTextBox->txtbLineCount - 1);
     while((inTextBox->txtbLines[tempLinePtr] >= inTextBox->txtbSelStart) && (tempLinePtr > 0))
          tempLinePtr--;
     inTextBox->txtbSelRow = tempLinePtr;
}

int txtbGetCol(TextBox* inTextBox, int inPos) {
     int tempLinePtr = (inTextBox->txtbLineCount - 1);
     while((inTextBox->txtbLines[tempLinePtr] >= inPos) && (tempLinePtr > 0))
          tempLinePtr--;
     return (inPos - inTextBox->txtbLines[tempLinePtr] - 1);
}

int txtbGetRow(TextBox* inTextBox, int inPos) {
     int tempLinePtr = (inTextBox->txtbLineCount - 1);
     while((inTextBox->txtbLines[tempLinePtr] >= inPos) && (tempLinePtr > 0))
          tempLinePtr--;
     return tempLinePtr;
}

void txtbDelete(TextBox* inTextBox) {
     if(inTextBox->txtbText[inTextBox->txtbSelStart] == 0)
          return;
     int tempDelLen = inTextBox->txtbSelEnd - inTextBox->txtbSelStart;
     if(tempDelLen < 0) {
          tempDelLen = 0 - tempDelLen;
          inTextBox->txtbSelStart = inTextBox->txtbSelEnd;
     }
     if(tempDelLen < 1)
          tempDelLen = 1;
     int tempPtr = inTextBox->txtbSelStart + tempDelLen;
     if(inTextBox->txtbText[tempPtr] == 0)
          inTextBox->txtbText[inTextBox->txtbSelStart] = 0;
     while(inTextBox->txtbText[tempPtr] != 0) {
          inTextBox->txtbText[tempPtr - tempDelLen] = inTextBox->txtbText[tempPtr];
          inTextBox->txtbText[tempPtr] = 0;
          tempPtr++;
     }
     inTextBox->txtbSelEnd = inTextBox->txtbSelStart;
     txtbRecacheLines(inTextBox);
     inTextBox->txtbChanged = 1;
}

void txtbBackSpace(TextBox* inTextBox) {
     if(inTextBox->txtbSelEnd != inTextBox->txtbSelStart) {
          txtbDelete(inTextBox);
     } else {
          if(inTextBox->txtbSelStart <= 0)
               return;
          inTextBox->txtbSelStart--;
          inTextBox->txtbSelEnd--;
          txtbDelete(inTextBox);
     }
     txtbCursorRow(inTextBox); txtbCursorCol(inTextBox);
}

void txtbInsertChar(TextBox* inTextBox, unsigned char inChar) {
     if(inTextBox->txtbSelStart != inTextBox->txtbSelEnd) {
          txtbDelete(inTextBox);
     } else if(inTextBox->txtbOvrMode != 0) {
          if(inTextBox->txtbText[inTextBox->txtbSelStart] == 0)
               return;
          inTextBox->txtbText[inTextBox->txtbSelStart] = inChar;
          inTextBox->txtbSelStart++;
          inTextBox->txtbSelEnd++;
          inTextBox->txtbChanged = 1;
          txtbRecacheLines(inTextBox);
          txtbCursorRow(inTextBox); txtbCursorCol(inTextBox);
          return;
     }
     int tempPtr = inTextBox->txtbLines[inTextBox->txtbLineCount - 1];
     if(tempPtr < 0)
          tempPtr = 0;
     while(inTextBox->txtbText[tempPtr] != 0)
          tempPtr++;
     inTextBox->txtbText[tempPtr + 1] = 0;
     while(tempPtr != inTextBox->txtbSelStart) {
          inTextBox->txtbText[tempPtr] =  inTextBox->txtbText[tempPtr - 1];
          tempPtr--;
     }
     inTextBox->txtbText[inTextBox->txtbSelStart] = inChar;
     inTextBox->txtbSelStart++;
     inTextBox->txtbSelEnd++;
     inTextBox->txtbChanged = 1;
     txtbRecacheLines(inTextBox);
     txtbCursorRow(inTextBox); txtbCursorCol(inTextBox);
}

void txtbPageUp(TextBox* inTextBox) {
     int tempInt;
     for(tempInt = 0; tempInt < TEXTBOX_PAGESCROLL; tempInt++)
          txtbCursorMoveUp(inTextBox);
}

void txtbPageDown(TextBox* inTextBox) {
     int tempInt;
     for(tempInt = 0; tempInt < TEXTBOX_PAGESCROLL; tempInt++)
          txtbCursorMoveDown(inTextBox);
}

void txtbHome(TextBox* inTextBox) {
     if(inTextBox->txtbSelCol == 0)
           return;
     inTextBox->txtbSelStart -= inTextBox->txtbSelCol;
     if(inTextBox->txtbSelMode == 0)
          inTextBox->txtbSelEnd = inTextBox->txtbSelStart;
     txtbCursorRow(inTextBox); txtbCursorCol(inTextBox);
}

void txtbEnd(TextBox* inTextBox) {
     int tempPtr = inTextBox->txtbSelStart;
     while((inTextBox->txtbText[tempPtr] != ASCII_LF) && (inTextBox->txtbText[tempPtr] != 0)) {
          inTextBox->txtbSelStart++;
          tempPtr++;
     }
     if(inTextBox->txtbSelMode == 0)
          inTextBox->txtbSelEnd = inTextBox->txtbSelStart;
     txtbCursorRow(inTextBox); txtbCursorCol(inTextBox);
}

void txtbFree(TextBox* inTextBox) {
     if(inTextBox->txtbText != NULL)
          free(inTextBox->txtbText);
     if(inTextBox->txtbLines != NULL)
          free(inTextBox->txtbLines);
     free(inTextBox);
}

void txtbCopy(TextBox* inTextBox) {
     if(inTextBox->txtbSelStart == inTextBox->txtbSelEnd)
          return;
     int tempSelLen = inTextBox->txtbSelEnd - inTextBox->txtbSelStart;
     if(tempSelLen < 0)
          tempSelLen = 0 - tempSelLen;
     int tempSelStart = MIN(inTextBox->txtbSelStart, inTextBox->txtbSelEnd);
     copybufSet(&inTextBox->txtbText[tempSelStart], tempSelLen);
}

void txtbCut(TextBox* inTextBox) {
     txtbCopy(inTextBox);
     txtbDelete(inTextBox);
}

void txtbPaste(TextBox* inTextBox) {
     unsigned char* tempString = copybufGet();
     if(tempString == NULL)
          return;
     int tempPtr = 0;
     while(tempString[tempPtr] != 0) {
          txtbInsertChar(inTextBox, tempString[tempPtr]);
          tempPtr++;
     }
}

void txtbSelectAll(TextBox* inTextBox) {
     inTextBox->txtbSelStart = 0;
     inTextBox->txtbSelEnd = strlen((char*)inTextBox->txtbText);
}

void txtbGotoLine(TextBox* inTextBox, int inLine) {
     if(inLine <= 0) {
          inTextBox->txtbSelStart = 0;
          inTextBox->txtbSelEnd = 0;
          return;
     }
     if(inLine > inTextBox->txtbLineCount)
          inLine = inTextBox->txtbLineCount;
     inLine--;
     inTextBox->txtbSelStart = inTextBox->txtbLines[inLine] + 1;
     inTextBox->txtbSelEnd = inTextBox->txtbSelStart;
}

void txtbReplace(TextBox* inTextBox, char* inFind, char* inReplace) {
     int tempPtr = 0;
     int i = 0;
     while(inTextBox->txtbText[tempPtr] != 0) {
          if(memcmp(&inTextBox->txtbText[tempPtr], inFind, strlen(inFind)) == 0) {
               inTextBox->txtbSelStart = tempPtr;
               inTextBox->txtbSelEnd = inTextBox->txtbSelStart + strlen(inFind);
               for(i = 0; i < strlen(inReplace); i++)
                    txtbInsertChar(inTextBox, inReplace[i]);
               txtbRecacheLines(inTextBox);
          }
          tempPtr++;
          if(inTextBox->txtbText[tempPtr + strlen(inFind) - 1] == 0)
               break;
     }
}

int txtbFindNext(TextBox* inTextBox, char* inFind) {
     int tempPtr = inTextBox->txtbSelStart;
     while(inTextBox->txtbText[tempPtr] != 0) {
          if(memcmp(&inTextBox->txtbText[tempPtr], inFind, strlen(inFind)) == 0) {
               inTextBox->txtbSelEnd = tempPtr;
               inTextBox->txtbSelStart = inTextBox->txtbSelEnd + strlen(inFind);
               txtbCursorRow(inTextBox);
               return inTextBox->txtbSelRow;
          }
          tempPtr++;
          if(inTextBox->txtbText[tempPtr + strlen(inFind) - 1] == 0)
               break;
     }
     if(dlgConfirm("Searched to the end", "search again from the start?") != 0) {
          inTextBox->txtbSelStart = 0;
          return txtbFindNext(inTextBox, inFind);
     }
     return -1;
}

int txtbFindPrev(TextBox* inTextBox, char* inFind) {
     int tempPtr = inTextBox->txtbSelStart;
     while(tempPtr > 0) {
          if(memcmp(&inTextBox->txtbText[tempPtr], inFind, strlen(inFind)) == 0) {
               inTextBox->txtbSelStart = tempPtr;
               inTextBox->txtbSelEnd = inTextBox->txtbSelStart + strlen(inFind);
               txtbCursorRow(inTextBox);
               return inTextBox->txtbSelRow;
          }
          tempPtr--;
          if(inTextBox->txtbText[tempPtr + strlen(inFind) - 1] == 0)
               break;
     }
     if(dlgConfirm("Searched to the start", "search again from the end?") != 0) {
          inTextBox->txtbSelStart = inTextBox->txtbLines[inTextBox->txtbLineCount - 1];
          while(inTextBox->txtbText[inTextBox->txtbSelStart] != 0)
               inTextBox->txtbSelStart++;
          return txtbFindPrev(inTextBox, inFind);
     }
     return -1;
}
