#ifndef TEXTBOX_H
#define TEXTBOX_H

#include <psptypes.h>
#include "font.h"
#include "keyboard.h"

#define TEXTBOX_TABSPACING 5
#define TEXTBOX_SIZE 8388608
#define TEXTBOX_PAGESCROLL 12

typedef struct {
     int txtbTop, txtbLeft;
     unsigned int txtbWidth, txtbHeight;

     Font* txtbFont;
     unsigned char* txtbText;
     
     int* txtbLines;
     int txtbLineCount;
     int txtbLineMaxWidth;
     
     unsigned int txtbSize;
     char txtbPath[768];
     
     int txtbScrollH, txtbScrollV;
     unsigned int txtbSelStart, txtbSelEnd;
     unsigned int txtbSelCol, txtbSelRow;
     unsigned char txtbAnim, txtbChanged, txtbFocus;
     unsigned char txtbSelMode, txtbOvrMode;
} TextBox;

extern TextBox* txtbCreate(unsigned int inSize, Font* inFont, int inLeft, int inTop, unsigned int inWidth, unsigned int inHeight);
extern void txtbTick(TextBox* inTextBox);
extern void txtbRecacheLines(TextBox* inTextBox);
extern int txtbLineWidth(TextBox* inTextBox, int inLine);
extern void txtbDraw(TextBox* inTextBox);
extern void txtbAddFromKeyboard(TextBox* inTextBox, Keyboard* inKeyboard);
extern void txtbBackSpace(TextBox* inTextBox);
extern void txtbClear(TextBox* inTextBox);
extern void txtbOpen(TextBox* inTextBox, char* inPath, char inQuiet);
extern void txtbSave(TextBox* inTextBox, char* inPath, char inQuiet);
extern void txtbCursorMoveUp(TextBox* inTextBox);
extern void txtbCursorMoveDown(TextBox* inTextBox);
extern void txtbCursorMoveLeft(TextBox* inTextBox);
extern void txtbCursorMoveRight(TextBox* inTextBox);
extern void txtbCursorRow(TextBox* inTextBox);
extern void txtbCursorCol(TextBox* inTextBox);
extern int txtbGetCol(TextBox* inTextBox, int inPos);
extern int txtbGetRow(TextBox* inTextBox, int inPos);
extern void txtbDelete(TextBox* inTextBox);
extern void txtbPageUp(TextBox* inTextBox);
extern void txtbPageDown(TextBox* inTextBox);
extern void txtbHome(TextBox* inTextBox);
extern void txtbEnd(TextBox* inTextBox);
extern void txtbInsertChar(TextBox* inTextBox, unsigned char inChar);
extern void txtbFree(TextBox* inTextBox);
extern void txtbCopy(TextBox* inTextBox);
extern void txtbCut(TextBox* inTextBox);
extern void txtbPaste(TextBox* inTextBox);
extern void txtbSelectAll(TextBox* inTextBox);
extern void txtbGotoLine(TextBox* inTextBox, int inLine);
extern void txtbReplace(TextBox* inTextBox, char* inFind, char* inReplace);
extern int txtbFindNext(TextBox* inTextBox, char* inFind);
extern int txtbFindPrev(TextBox* inTextBox, char* inFind);

#endif
